@@ -26,7 +26,7 @@
 #include "global.h"
 #include "md5.h"
 #include <sys/types.h>
-#if !defined(WIN32) && !defined(WIN64)
+#if !defined(_WIN32)
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
@@ -40,6 +40,7 @@
 #include <stdio.h>
 #include <unistd.h>
 
+
 #define BLOCK_SIZE 1024
 
 /* Update DB Host & HTTP GET Request formats:
@@ -107,33 +108,56 @@ const char * GeoIP_get_error_message(int i) {
   }  
 }
 int GeoIP_fprintf(int (*f)(FILE *, char *),FILE *fp, const char *str, ...) {
-	va_list ap;
-	int rc;
+  va_list ap;
+  int rc;
   char * f_str;
+  int silence;
+
   if ( f == NULL )
     return 0;
-	va_start(ap, str);
-  vasprintf(&f_str, str, ap);
+  va_start(ap, str);
+#if defined(HAVE_VASPRINTF)
+  silence = vasprintf(&f_str, str, ap);
+#elif defined (HAVE_VSNPRINTF)
+  f_str = malloc(4096);
+  if ( f_str )
+    silence = vsnprintf(f_str, 4096, str, ap);
+#else
+  f_str = malloc(4096);
+  if ( f_str )
+    silence = vsprintf(f_str, str, ap);
+#endif
   va_end(ap);
   if (  f_str == NULL )
     return -1;
   rc = (*f)(fp, f_str);
   free(f_str);
-	return(rc);
+  return(rc);
 }
 
 void GeoIP_printf(void (*f)(char *), const char *str,...) {
   va_list params;
-	char * f_str;
+  char * f_str;
+  int silence;
   if (f == NULL)
-		return;
+    return;
   va_start(params, str);
-  vasprintf(&f_str, str, params);
+#if defined(HAVE_VASPRINTF)
+  silence = vasprintf(&f_str, str, params);
+#elif defined (HAVE_VSNPRINTF)
+  f_str = malloc(4096);
+  if ( f_str )
+    silence = vsnprintf(f_str, 4096, str, params);
+#else
+  f_str = malloc(4096);
+  if ( f_str )
+    silence = vsprintf(f_str, str, params);
+#endif
   va_end(params);
   if ( f_str == NULL )
     return;
-	(*f)(f_str);
-	free(f_str);
+  (*f)(f_str);
+  free(f_str);
 }
 
 /* Support HTTP Proxy Host
@@ -223,7 +247,7 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 	char block[BLOCK_SIZE];
 	int block_size = BLOCK_SIZE;
 	size_t len;
-
+	size_t written;
 	_GeoIP_setup_dbfilename();
 
 	/* get MD5 of current GeoIP database file */
@@ -269,14 +293,15 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 	if (connect(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr))< 0)
 		return GEOIP_CONNECTION_ERR;
 
-	request_uri = malloc(sizeof(char) * (strlen(license_key) + strlen(GeoIPHTTPRequest)+36));
+	request_uri = malloc(sizeof(char) * (strlen(license_key) + strlen(GeoIPHTTPRequest)
+                              + strlen(GeoIPProxyHTTP) + strlen(GeoIPProxiedHost) + 36 + 1));
 	if (request_uri == NULL)
 		return GEOIP_OUT_OF_MEMORY_ERR;
 	sprintf(request_uri,GeoIPHTTPRequest,GeoIPProxyHTTP,GeoIPProxiedHost,license_key, hex_digest);
 	send(sock, request_uri, strlen(request_uri),0);
 	free(request_uri);
 
-	buf = malloc(sizeof(char) * block_size);
+	buf = malloc(sizeof(char) * block_size + 1);
 	if (buf == NULL)
 		return GEOIP_OUT_OF_MEMORY_ERR;
 
@@ -293,12 +318,19 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 			return GEOIP_SOCKET_READ_ERR;
 		}
 		offset += amt;
-		buf = realloc(buf, offset+block_size);
+		buf = realloc(buf, offset+block_size + 1);
 		if (buf == NULL)
 			return GEOIP_OUT_OF_MEMORY_ERR;
 	}
 
-	compr = strstr(buf, "\r\n\r\n") + 4;
+        buf[offset]=0;
+	compr = strstr(buf, "\r\n\r\n");
+        if ( compr == NULL ) {
+   		free(buf);
+		return GEOIP_INVALID_SERVER_RESPONSE;
+        }
+        /* skip searchstr  "\r\n\r\n" */
+        compr += 4;
 	comprLen = offset + buf - compr;
 
 	if (strstr(compr, "License Key Invalid") != NULL) {
@@ -334,10 +366,13 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 		return GEOIP_GZIP_IO_ERR;
 	}
 
-	fwrite(compr, 1, comprLen, comp_fh);
+	written = fwrite(compr, 1, comprLen, comp_fh);
 	fclose(comp_fh);
 	free(buf);
 
+        if ( written != comprLen )
+		return GEOIP_GZIP_IO_ERR;
+
 	if (verbose == 1)
 		GeoIP_printf(f,"Done\n");
 
@@ -369,7 +404,12 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 		if (amt == 0) {
 			break;
 		}
-		fwrite(block,1,amt,gi_fh);
+		if ( fwrite(block,1,amt,gi_fh) != amt ){
+			free(file_path_test);
+			fclose(gi_fh);
+			gzclose(gz_fh);
+			return GEOIP_GZIP_READ_ERR;
+		}
 	}
 	gzclose(gz_fh);
 	unlink(file_path_gz);
@@ -472,6 +512,7 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	int block_size = BLOCK_SIZE;
 	size_t len;
 	size_t request_uri_len;
+	size_t size;
 
 	hostlist = GeoIP_get_host_or_proxy();
 
@@ -496,7 +537,9 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	
 	if (connect(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr))< 0)
 		return GEOIP_CONNECTION_ERR;
-	request_uri = malloc(sizeof(char) * (strlen(license_key) + strlen(GeoIPHTTPRequestMD5)+1036));
+	request_uri = malloc(sizeof(char) * (strlen(GeoIPHTTPRequestFilename) 
+                                             + strlen(GeoIPProxyHTTP) + strlen(GeoIPProxiedHost)
+                                             + strlen(data_base_type) + strlen(GeoIPUpdateHost) + 1));
 	if (request_uri == NULL)
 		return GEOIP_OUT_OF_MEMORY_ERR;
 
@@ -525,7 +568,13 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	}
 	buf[offset] = 0;
 	offset = 0;
-	tmpstr = strstr(buf, "\r\n\r\n") + 4;
+	tmpstr = strstr(buf, "\r\n\r\n");
+        if ( tmpstr == NULL ) {
+   		free(buf);
+		return GEOIP_INVALID_SERVER_RESPONSE;
+        }
+        /* skip searchstr  "\r\n\r\n" */
+        tmpstr += 4;
 	if (tmpstr[0] == '.' || strchr(tmpstr, '/') != NULL) {
 		free(buf);
 		return GEOIP_INVALID_SERVER_RESPONSE;
@@ -576,7 +625,10 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 			free(geoipfilename);
 			return GEOIP_CONNECTION_ERR;
 		}
-		request_uri = malloc(sizeof(char) * (strlen(license_key) + strlen(GeoIPHTTPRequestMD5)+1036));
+		request_uri = malloc(sizeof(char) * (strlen(GeoIPHTTPRequestClientIP) 
+                                                     + strlen(GeoIPProxyHTTP) 
+                                                     + strlen(GeoIPProxiedHost)
+                                                     + strlen(GeoIPUpdateHost) + 1 ));
 		if (request_uri == NULL) {
 			free(geoipfilename);
 			return GEOIP_OUT_OF_MEMORY_ERR;
@@ -625,11 +677,9 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	ipaddress = client_ipaddr[0];
 
 	/* make a md5 sum of ip address and license_key and store it in hex_digest2 */
-	request_uri_len = sizeof(char) * 2036;
-	request_uri = malloc(request_uri_len);
 	md5_init(&context2);
-	md5_write (&context2, license_key, 12);//add license key to the md5 sum
-	md5_write (&context2, ipaddress, strlen(ipaddress));//add ip address to the md5 sum
+	md5_write (&context2, (byte *)license_key, 12);//add license key to the md5 sum
+	md5_write (&context2, (byte *)ipaddress, strlen(ipaddress));//add ip address to the md5 sum
 	md5_final (&context2);
 	memcpy(digest2,context2.buf,16);
 	for (i = 0; i < 16; i++)
@@ -650,6 +700,10 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	sa.sin_family = AF_INET;
 	if (connect(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr))< 0)
 		return GEOIP_CONNECTION_ERR;
+	request_uri_len = sizeof(char) * 2036;
+	request_uri = malloc(request_uri_len);
+	if (request_uri == NULL)
+	        return GEOIP_OUT_OF_MEMORY_ERR;
 	snprintf(request_uri, request_uri_len, GeoIPHTTPRequestMD5,GeoIPProxyHTTP,GeoIPProxiedHost,hex_digest,hex_digest2,user_id,data_base_type);
 	send(sock, request_uri, strlen(request_uri),0);
 	if (verbose == 1) {
@@ -715,7 +769,7 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	strcpy(file_path_gz,geoipfilename);
 	strcat(file_path_gz,".gz");
 	if (verbose == 1) {
-    GeoIP_printf(f, "%s%s", SavingGzip, file_path_gz );
+    GeoIP_printf(f, SavingGzip, file_path_gz );
 	}
 	comp_fh = fopen(file_path_gz, "wb");
 
@@ -725,9 +779,12 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 		return GEOIP_GZIP_IO_ERR;
 	}
 
-	fwrite(compr, 1, comprLen, comp_fh);
+	size = fwrite(compr, 1, comprLen, comp_fh);
 	fclose(comp_fh);
 	free(buf);
+        if ( size != comprLen ) {
+		return GEOIP_GZIP_IO_ERR;
+	}
 
 	if (verbose == 1) {
 		GeoIP_printf(f, "download data to a gz file named %s \n",file_path_gz);
@@ -764,7 +821,9 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 		if (amt == 0) {
 			break;
 		}
-		fwrite(block,1,amt,gi_fh);
+		if ( amt != fwrite(block,1,amt,gi_fh) ){
+			return GEOIP_GZIP_IO_ERR;
+		}
 	}
 	gzclose(gz_fh);
 	unlink(file_path_gz);
